@@ -6,19 +6,19 @@ use warnings;
 package DictParser;
 use base qw( Parser::MGC );
 
+use Feature::Compat::Try;
+
 sub parse
 {
    my $self = shift;
 
    $self->any_of(
-      sub { $self->token_int },
+      'token_int',
+      'token_string',
 
-      sub { $self->token_string },
+      sub { $self->committed_scope_of( "{", 'parse_dict', "}" ) },
 
-      sub { $self->scope_of( "{",
-               sub { $self->commit; $self->parse_dict },
-            "}" );
-      },
+      sub { $self->commit; $self->fail( "Expected integer, string, or dictionary" ) },
    );
 }
 
@@ -31,6 +31,7 @@ sub parse_dict
       my $key = $self->token_ident;
 
       $self->expect( ":" );
+      $self->commit;
 
       $ret{$key} = $self->parse;
    } );
@@ -38,16 +39,19 @@ sub parse_dict
    return \%ret
 }
 
-use Data::Dump qw( pp );
+use Data::Dumper;
 
 if( !caller ) {
    my $parser = __PACKAGE__->new;
 
    while( defined( my $line = <STDIN> ) ) {
-      my $ret = eval { $parser->from_string( $line ) };
-      print $@ and next if $@;
-
-      print pp( $ret ) . "\n";
+      try {
+         my $ret = $parser->from_string( $line );
+         print Dumper( $ret );
+      }
+      catch ( $e ) {
+         print $e;
+      }
    }
 }
 
