@@ -6,6 +6,8 @@ use warnings;
 package PodParser;
 use base qw( Parser::MGC );
 
+use Feature::Compat::Try;
+
 sub parse
 {
    my $self = shift;
@@ -13,9 +15,8 @@ sub parse
    $self->sequence_of(
       sub { $self->any_of(
 
-         sub { my $tag = $self->expect( qr/[A-Z](?=<)/ );
+         sub { my ( undef, $tag, $delim ) = $self->expect( qr/([A-Z])(<+)/ );
                $self->commit;
-               my $delim = $self->expect( qr/<+/ );
                +{ $tag => $self->scope_of( undef, \&parse, ">" x length $delim ) }; },
 
          sub { $self->substring_before( qr/[A-Z]</ ) },
@@ -23,16 +24,19 @@ sub parse
    );
 }
 
-use Data::Dump qw( pp );
+use Data::Dumper;
 
 if( !caller ) {
    my $parser = __PACKAGE__->new;
 
    while( defined( my $line = <STDIN> ) ) {
-      my $ret = eval { $parser->from_string( $line ) };
-      print $@ and next if $@;
-
-      print pp( $ret ) . "\n";
+      try {
+         my $ret = $parser->from_string( $line );
+         print Dumper( $ret );
+      }
+      catch ( $e ) {
+         print $e;
+      }
    }
 }
 
