@@ -34,10 +34,7 @@ sub parse_node
    my $self = shift;
 
    # A "node" is either an XML element subtree or plaintext
-   $self->any_of(
-      \&parse_plaintext,
-      \&parse_element,
-   );
+   $self->any_of( 'parse_plaintext', 'parse_element' );
 }
 
 sub parse_plaintext
@@ -60,7 +57,7 @@ sub parse_element
 
    return XmlParser::Node::Element->new( $tag->{name}, $tag->{attrs} ) if $tag->{selfclose};
 
-   my $childlist = $self->sequence_of( \&parse_node );
+   my $childlist = $self->sequence_of( 'parse_node' );
 
    $self->parse_close_tag->{name} eq $tag->{name}
       or $self->fail( "Expected $tag->{name} to be closed" );
@@ -75,7 +72,7 @@ sub parse_tag
    $self->expect( '<' );
    my $tagname = $self->token_ident;
 
-   my $attrs = $self->sequence_of( \&parse_tag_attr );
+   my $attrs = $self->sequence_of( 'parse_tag_attr' );
 
    my $selfclose = $self->maybe_expect( '/' );
    $self->expect( '>' );
